/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.ow2.util.log;

import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.logging.ErrorManager;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

/**
 * Handler that display info message in System.out and error in System.err
 * because JDK implementation prints all messages in System.err and in Eclipse,
 * for example, INFO messages are in red !
 * @author Florent Benoit
 */
public class JDKConsoleHandler extends Handler {

    /**
     * Writer used for traces INFO, FINE, FINEST, etc.
     */
    private Writer infoOutputWriter;

    /**
     * Writer used for WARNING, SEVERE, FATAL.
     */
    private Writer errOutputWriter;

    /**
     * Creates a new handler with two appender, System.out and System.err.
     */
    public JDKConsoleHandler() {
        super();
        infoOutputWriter = new OutputStreamWriter(System.out);
        errOutputWriter = new OutputStreamWriter(System.err);
    }

    /**
     * Prints the record on the correct writer.
     * @param logRecord the given record with all info to log.
     */
    @Override
    public void publish(final LogRecord logRecord) {
        // Do nothing if not loggable
        if (!isLoggable(logRecord)) {
            return;
        }

        // Format the record
        String msg = null;
        try {
            msg = getFormatter().format(logRecord);
        } catch (Exception ex) {
            reportError(null, ex, ErrorManager.FORMAT_FAILURE);
            return;
        }

        // Get the correct writer
        Writer writer = null;
        if (logRecord.getLevel().intValue() >= Level.WARNING.intValue()) {
            writer = errOutputWriter;
        } else {
            writer = infoOutputWriter;
        }

        // Write the message
        try {
            writer.write(msg);
        } catch (Exception ex) {
            reportError(null, ex, ErrorManager.WRITE_FAILURE);
        }

        // Flush the current writer
        flush(writer);
    }

    /**
     * Flush the given writer.
     * @param writer the writer to flush.
     */
    private void flush(final Writer writer) {
        try {
            writer.flush();
        } catch (Exception ex) {
            reportError(null, ex, ErrorManager.FLUSH_FAILURE);
        }
    }

    /**
     * Flush all the current writers (info and error).
     */
    @Override
    public void flush() {
        flush(errOutputWriter);
        flush(infoOutputWriter);
    }

    /**
     * Close the handler by flushing all writers.
     */
    @Override
    public void close() {
        flush();
    }
}
